'use strict';

var anchors = require('../doc/anchors.js');

var Node = require('../nodes/Node.js');

var stringifyComment = require('./stringifyComment.js');

var stringifyString = require('./stringifyString.js');

function createStringifyContext(doc, options) {
  const opt = Object.assign({
    blockQuote: true,
    commentString: stringifyComment.stringifyComment,
    defaultKeyType: null,
    defaultStringType: 'PLAIN',
    directives: null,
    doubleQuotedAsJSON: false,
    doubleQuotedMinMultiLineLength: 40,
    falseStr: 'false',
    indentSeq: true,
    lineWidth: 80,
    minContentWidth: 20,
    nullStr: 'null',
    simpleKeys: false,
    singleQuote: null,
    trueStr: 'true',
    verifyAliasOrder: true
  }, doc.schema.toStringOptions, options);
  let inFlow;

  switch (opt.collectionStyle) {
    case 'block':
      inFlow = false;
      break;

    case 'flow':
      inFlow = true;
      break;

    default:
      inFlow = null;
  }

  return {
    anchors: new Set(),
    doc,
    indent: '',
    indentStep: typeof opt.indent === 'number' ? ' '.repeat(opt.indent) : '  ',
    inFlow,
    options: opt
  };
}

function getTagObject(tags, item) {
  if (item.tag) {
    var _match$find;

    const match = tags.filter(t => t.tag === item.tag);
    if (match.length > 0) return (_match$find = match.find(t => t.format === item.format)) !== null && _match$find !== void 0 ? _match$find : match[0];
  }

  let tagObj = undefined;
  let obj;

  if (Node.isScalar(item)) {
    var _match$find2;

    obj = item.value;
    const match = tags.filter(t => {
      var _t$identify;

      return (_t$identify = t.identify) === null || _t$identify === void 0 ? void 0 : _t$identify.call(t, obj);
    });
    tagObj = (_match$find2 = match.find(t => t.format === item.format)) !== null && _match$find2 !== void 0 ? _match$find2 : match.find(t => !t.format);
  } else {
    obj = item;
    tagObj = tags.find(t => t.nodeClass && obj instanceof t.nodeClass);
  }

  if (!tagObj) {
    var _obj$constructor$name, _obj, _obj$constructor;

    const name = (_obj$constructor$name = (_obj = obj) === null || _obj === void 0 ? void 0 : (_obj$constructor = _obj.constructor) === null || _obj$constructor === void 0 ? void 0 : _obj$constructor.name) !== null && _obj$constructor$name !== void 0 ? _obj$constructor$name : typeof obj;
    throw new Error(`Tag not resolved for ${name} value`);
  }

  return tagObj;
} // needs to be called before value stringifier to allow for circular anchor refs


function stringifyProps(node, tagObj, {
  anchors: anchors$1,
  doc
}) {
  if (!doc.directives) return '';
  const props = [];
  const anchor = (Node.isScalar(node) || Node.isCollection(node)) && node.anchor;

  if (anchor && anchors.anchorIsValid(anchor)) {
    anchors$1.add(anchor);
    props.push(`&${anchor}`);
  }

  const tag = node.tag ? node.tag : tagObj.default ? null : tagObj.tag;
  if (tag) props.push(doc.directives.tagString(tag));
  return props.join(' ');
}

function stringify(item, ctx, onComment, onChompKeep) {
  var _ctx$indentAtStart;

  if (Node.isPair(item)) return item.toString(ctx, onComment, onChompKeep);

  if (Node.isAlias(item)) {
    var _ctx$resolvedAliases;

    if (ctx.doc.directives) return item.toString(ctx);

    if ((_ctx$resolvedAliases = ctx.resolvedAliases) !== null && _ctx$resolvedAliases !== void 0 && _ctx$resolvedAliases.has(item)) {
      throw new TypeError(`Cannot stringify circular structure without alias nodes`);
    } else {
      if (ctx.resolvedAliases) ctx.resolvedAliases.add(item);else ctx.resolvedAliases = new Set([item]);
      item = item.resolve(ctx.doc);
    }
  }

  let tagObj = undefined;
  const node = Node.isNode(item) ? item : ctx.doc.createNode(item, {
    onTagObj: o => tagObj = o
  });
  if (!tagObj) tagObj = getTagObject(ctx.doc.schema.tags, node);
  const props = stringifyProps(node, tagObj, ctx);
  if (props.length > 0) ctx.indentAtStart = ((_ctx$indentAtStart = ctx.indentAtStart) !== null && _ctx$indentAtStart !== void 0 ? _ctx$indentAtStart : 0) + props.length + 1;
  const str = typeof tagObj.stringify === 'function' ? tagObj.stringify(node, ctx, onComment, onChompKeep) : Node.isScalar(node) ? stringifyString.stringifyString(node, ctx, onComment, onChompKeep) : node.toString(ctx, onComment, onChompKeep);
  if (!props) return str;
  return Node.isScalar(node) || str[0] === '{' || str[0] === '[' ? `${props} ${str}` : `${props}\n${ctx.indent}${str}`;
}

exports.createStringifyContext = createStringifyContext;
exports.stringify = stringify;